@@ -105,7 +105,7 @@ struct elf_module_info
 #define ELF_SHT_FINI_ARRAY     15    /* Array of destructors */
 #define ELF_SHT_PREINIT_ARRAY  16    /* Array of pre-constructors */
 #define ELF_SHT_GROUP          17    /* Section group */
-#define ELF_SHT_SYMTAB_SHNDX   18    /* Extended section indeces */
+#define ELF_SHT_SYMTAB_SHNDX   18    /* Extended section indices */
 #define ELF_SHT_NUM            19    /* Number of defined types.  */
 
 /* Legal values for ST_TYPE subfield of st_info (symbol type).  */
@@ -1435,7 +1435,7 @@ static BOOL elf_search_and_load_file(struct process* pcs, const WCHAR* filename,
         load_elf.dyn_addr    = dyn_addr;
         load_elf.elf_info    = elf_info;
 
-        ret = search_unix_path(filename, getenv("LD_LIBRARY_PATH"), elf_load_file_cb, &load_elf)
+        ret = search_unix_path(filename, process_getenv(pcs, L"LD_LIBRARY_PATH"), elf_load_file_cb, &load_elf)
             || search_dll_path(pcs, filename, elf_load_file_cb, &load_elf);
     }
 
