@@ -45,16 +45,19 @@ static BOOL refdevice = FALSE;
 static HRESULT (WINAPI *pDirectDrawCreateEx)(GUID *driver_guid,
         void **ddraw, REFIID interface_iid, IUnknown *outer);
 
+static BOOL compare_uint(unsigned int x, unsigned int y, unsigned int max_diff)
+{
+    unsigned int diff = x > y ? x - y : y - x;
+
+    return diff <= max_diff;
+}
+
 static BOOL color_match(D3DCOLOR c1, D3DCOLOR c2, BYTE max_diff)
 {
-    if (abs((c1 & 0xff) - (c2 & 0xff)) > max_diff) return FALSE;
-    c1 >>= 8; c2 >>= 8;
-    if (abs((c1 & 0xff) - (c2 & 0xff)) > max_diff) return FALSE;
-    c1 >>= 8; c2 >>= 8;
-    if (abs((c1 & 0xff) - (c2 & 0xff)) > max_diff) return FALSE;
-    c1 >>= 8; c2 >>= 8;
-    if (abs((c1 & 0xff) - (c2 & 0xff)) > max_diff) return FALSE;
-    return TRUE;
+    return compare_uint(c1 & 0xff, c2 & 0xff, max_diff)
+            && compare_uint((c1 >> 8) & 0xff, (c2 >> 8) & 0xff, max_diff)
+            && compare_uint((c1 >> 16) & 0xff, (c2 >> 16) & 0xff, max_diff)
+            && compare_uint((c1 >> 24) & 0xff, (c2 >> 24) & 0xff, max_diff);
 }
 
 static HRESULT WINAPI enum_z_fmt(DDPIXELFORMAT *fmt, void *ctx)
@@ -267,52 +270,6 @@ static void set_viewport_size(IDirect3DDevice7 *device)
     return;
 }
 
-static void clear_test(IDirect3DDevice7 *device)
-{
-    /* Tests the correctness of clearing parameters */
-    HRESULT hr;
-    D3DRECT rect[2];
-    D3DRECT rect_negneg;
-    DWORD color;
-
-    hr = IDirect3DDevice7_Clear(device, 0, NULL, D3DCLEAR_TARGET, 0xffffffff, 0.0, 0);
-    ok(hr == D3D_OK, "IDirect3DDevice7_Clear failed with %08x\n", hr);
-
-    /* Positive x, negative y */
-    U1(rect[0]).x1 = 0;
-    U2(rect[0]).y1 = 480;
-    U3(rect[0]).x2 = 320;
-    U4(rect[0]).y2 = 240;
-
-    /* Positive x, positive y */
-    U1(rect[1]).x1 = 0;
-    U2(rect[1]).y1 = 0;
-    U3(rect[1]).x2 = 320;
-    U4(rect[1]).y2 = 240;
-    /* Clear 2 rectangles with one call. Shows that a positive value is returned, but the negative rectangle
-     * is ignored, the positive is still cleared afterwards
-     */
-    hr = IDirect3DDevice7_Clear(device, 2, rect, D3DCLEAR_TARGET, 0xffff0000, 0.0, 0);
-    ok(hr == D3D_OK, "IDirect3DDevice7_Clear failed with %08x\n", hr);
-
-    /* negative x, negative y */
-    U1(rect_negneg).x1 = 640;
-    U2(rect_negneg).y1 = 240;
-    U3(rect_negneg).x2 = 320;
-    U4(rect_negneg).y2 = 0;
-    hr = IDirect3DDevice7_Clear(device, 1, &rect_negneg, D3DCLEAR_TARGET, 0xff00ff00, 0.0, 0);
-    ok(hr == D3D_OK, "IDirect3DDevice7_Clear failed with %08x\n", hr);
-
-    color = getPixelColor(device, 160, 360); /* lower left quad */
-    ok(color == 0x00ffffff, "Clear rectangle 3(pos, neg) has color %08x\n", color);
-    color = getPixelColor(device, 160, 120); /* upper left quad */
-    ok(color == 0x00ff0000, "Clear rectangle 1(pos, pos) has color %08x\n", color);
-    color = getPixelColor(device, 480, 360); /* lower right quad  */
-    ok(color == 0x00ffffff, "Clear rectangle 4(NULL) has color %08x\n", color);
-    color = getPixelColor(device, 480, 120); /* upper right quad */
-    ok(color == 0x00ffffff, "Clear rectangle 4(neg, neg) has color %08x\n", color);
-}
-
 static void fog_test(IDirect3DDevice7 *device)
 {
     HRESULT hr;
@@ -1681,7 +1638,6 @@ START_TEST(visual)
 
     /* Now run the tests */
     depth_clamp_test(Direct3DDevice);
-    clear_test(Direct3DDevice);
     fog_test(Direct3DDevice);
     offscreen_test(Direct3DDevice);
     test_blend(Direct3DDevice);
