@@ -329,8 +329,8 @@ static HRESULT RegExp_toString(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags, u
     TRACE("\n");
 
     if(!is_vclass(jsthis, JSCLASS_REGEXP)) {
-        FIXME("Not a RegExp\n");
-        return E_NOTIMPL;
+        WARN("Not a RegExp\n");
+        return JS_E_REGEXP_EXPECTED;
     }
 
     regexp = regexp_from_vdisp(jsthis);
@@ -671,10 +671,10 @@ HRESULT create_regexp(script_ctx_t *ctx, jsstr_t *src, DWORD flags, jsdisp_t **r
 
 HRESULT create_regexp_var(script_ctx_t *ctx, jsval_t src_arg, jsval_t *flags_arg, jsdisp_t **ret)
 {
-    unsigned flags, opt_len = 0;
+    unsigned flags = 0;
     const WCHAR *opt = NULL;
     jsstr_t *src;
-    HRESULT hres;
+    HRESULT hres = S_OK;
 
     if(is_object_instance(src_arg)) {
         jsdisp_t *obj;
@@ -693,33 +693,31 @@ HRESULT create_regexp_var(script_ctx_t *ctx, jsval_t src_arg, jsval_t *flags_arg
         }
     }
 
-    if(!is_string(src_arg)) {
-        FIXME("src_arg = %s\n", debugstr_jsval(src_arg));
-        return E_NOTIMPL;
-    }
-
-    src = get_string(src_arg);
-
-    if(flags_arg) {
-        jsstr_t *opt_str;
-
-        if(!is_string(*flags_arg)) {
-            FIXME("unimplemented for %s\n", debugstr_jsval(*flags_arg));
-            return E_NOTIMPL;
-        }
-
-        opt_str = get_string(*flags_arg);
-        opt = jsstr_flatten(opt_str);
-        if(!opt)
-            return E_OUTOFMEMORY;
-        opt_len = jsstr_length(opt_str);
-    }
-
-    hres = parse_regexp_flags(opt, opt_len, &flags);
+    if(is_undefined(src_arg))
+        src = jsstr_empty();
+    else
+        hres = to_string(ctx, src_arg, &src);
     if(FAILED(hres))
         return hres;
 
-    return create_regexp(ctx, src, flags, ret);
+    if(flags_arg && !is_undefined(*flags_arg)) {
+        jsstr_t *opt_str;
+
+        hres = to_string(ctx, *flags_arg, &opt_str);
+        if(SUCCEEDED(hres)) {
+            opt = jsstr_flatten(opt_str);
+            if(opt)
+                hres = parse_regexp_flags(opt, jsstr_length(opt_str), &flags);
+            else
+                hres = E_OUTOFMEMORY;
+            jsstr_release(opt_str);
+        }
+    }
+
+    if(SUCCEEDED(hres))
+        hres = create_regexp(ctx, src, flags, ret);
+    jsstr_release(src);
+    return hres;
 }
 
 HRESULT regexp_string_match(script_ctx_t *ctx, jsdisp_t *re, jsstr_t *jsstr, jsval_t *r)
@@ -953,12 +951,7 @@ static HRESULT RegExpConstr_value(script_ctx_t *ctx, vdisp_t *jsthis, WORD flags
         jsdisp_t *ret;
         HRESULT hres;
 
-        if(!argc) {
-            FIXME("no args\n");
-            return E_NOTIMPL;
-        }
-
-        hres = create_regexp_var(ctx, argv[0], argc > 1 ? argv+1 : NULL, &ret);
+        hres = create_regexp_var(ctx, argc ? argv[0] : jsval_undefined(), argc > 1 ? argv+1 : NULL, &ret);
         if(FAILED(hres))
             return hres;
 
