@@ -54,6 +54,7 @@
 #include "winnls.h"
 #include "winerror.h"
 #include "wincon.h"
+#include "wine/condrv.h"
 #include "wine/server.h"
 #include "wine/exception.h"
 #include "wine/unicode.h"
@@ -248,16 +249,10 @@ HANDLE WINAPI OpenConsoleW(LPCWSTR name, DWORD access, BOOL inherit, DWORD creat
  */
 BOOL WINAPI VerifyConsoleIoHandle(HANDLE handle)
 {
-    BOOL ret;
-
-    if (!is_console_handle(handle)) return FALSE;
-    SERVER_START_REQ(get_console_mode)
-    {
-	req->handle = console_handle_unmap(handle);
-	ret = !wine_server_call( req );
-    }
-    SERVER_END_REQ;
-    return ret;
+    IO_STATUS_BLOCK io;
+    DWORD mode;
+    return !NtDeviceIoControlFile( handle, NULL, NULL, NULL, &io, IOCTL_CONDRV_GET_MODE,
+                                   NULL, 0, &mode, sizeof(mode) );
 }
 
 /******************************************************************
@@ -409,6 +404,8 @@ static enum read_console_input_return read_console_input(HANDLE handle, PINPUT_R
 {
     int fd;
     enum read_console_input_return      ret;
+    int blocking = timeout != 0;
+    DWORD read_bytes;
 
     if ((fd = get_console_bare_fd(handle)) != -1)
     {
@@ -421,25 +418,10 @@ static enum read_console_input_return read_console_input(HANDLE handle, PINPUT_R
         close(fd);
         if (ret != rci_gotone) return ret;
     }
-    else
-    {
-        if (!VerifyConsoleIoHandle(handle)) return rci_error;
 
-        if (WaitForSingleObject(handle, timeout) != WAIT_OBJECT_0)
-            return rci_timeout;
-    }
-
-    SERVER_START_REQ( read_console_input )
-    {
-        req->handle = console_handle_unmap(handle);
-        req->flush = TRUE;
-        wine_server_set_reply( req, ir, sizeof(INPUT_RECORD) );
-        if (wine_server_call_err( req ) || !reply->read) ret = rci_error;
-        else ret = rci_gotone;
-    }
-    SERVER_END_REQ;
-
-    return ret;
+    if (!DeviceIoControl( handle, IOCTL_CONDRV_READ_INPUT, &blocking, sizeof(blocking), ir, sizeof(*ir), &read_bytes, NULL ))
+        return rci_error;
+    return read_bytes ? rci_gotone : rci_timeout;
 }
 
 
@@ -965,19 +947,16 @@ BOOL WINAPI WriteConsoleW(HANDLE hConsoleOutput, LPCVOID lpBuffer, DWORD nNumber
  */
 void CONSOLE_FillLineUniform(HANDLE hConsoleOutput, int i, int j, int len, LPCHAR_INFO lpFill)
 {
-    SERVER_START_REQ( fill_console_output )
-    {
-        req->handle    = console_handle_unmap(hConsoleOutput);
-        req->mode      = CHAR_INFO_MODE_TEXTATTR;
-        req->x         = i;
-        req->y         = j;
-        req->count     = len;
-        req->wrap      = FALSE;
-        req->data.ch   = lpFill->Char.UnicodeChar;
-        req->data.attr = lpFill->Attributes;
-        wine_server_call_err( req );
-    }
-    SERVER_END_REQ;
+    struct condrv_fill_output_params params;
+
+    params.mode  = CHAR_INFO_MODE_TEXTATTR;
+    params.x     = i;
+    params.y     = j;
+    params.count = len;
+    params.wrap  = FALSE;
+    params.ch    = lpFill->Char.UnicodeChar;
+    params.attr  = lpFill->Attributes;
+    DeviceIoControl( hConsoleOutput, IOCTL_CONDRV_FILL_OUTPUT, &params, sizeof(params), NULL, 0, NULL, NULL );
 }
 
 /******************************************************************
@@ -1089,15 +1068,9 @@ unsigned CONSOLE_GetNumHistoryEntries(void)
  */
 BOOL CONSOLE_GetEditionMode(HANDLE hConIn, int* mode)
 {
-    unsigned ret = 0;
-    SERVER_START_REQ(get_console_input_info)
-    {
-        req->handle = console_handle_unmap(hConIn);
-        if ((ret = !wine_server_call_err( req )))
-            *mode = reply->edition_mode;
-    }
-    SERVER_END_REQ;
-    return ret;
+    struct condrv_input_info info;
+    return DeviceIoControl( hConIn, IOCTL_CONDRV_GET_INPUT_INFO, NULL, 0, &info, sizeof(info), NULL, NULL )
+        ? info.edition_mode : 0;
 }
 
 /******************************************************************
@@ -1199,7 +1172,7 @@ BOOL CONSOLE_Init(RTL_USER_PROCESS_PARAMETERS *params)
      */
     if (!params->hStdInput || params->hStdInput == INVALID_HANDLE_VALUE)
         params->hStdInput = 0;
-    else if (VerifyConsoleIoHandle(console_handle_map(params->hStdInput)))
+    else if (VerifyConsoleIoHandle(params->hStdInput))
     {
         params->hStdInput = console_handle_map(params->hStdInput);
         save_console_mode(params->hStdInput);
@@ -1207,12 +1180,12 @@ BOOL CONSOLE_Init(RTL_USER_PROCESS_PARAMETERS *params)
 
     if (!params->hStdOutput || params->hStdOutput == INVALID_HANDLE_VALUE)
         params->hStdOutput = 0;
-    else if (VerifyConsoleIoHandle(console_handle_map(params->hStdOutput)))
+    else if (VerifyConsoleIoHandle(params->hStdOutput))
         params->hStdOutput = console_handle_map(params->hStdOutput);
 
     if (!params->hStdError || params->hStdError == INVALID_HANDLE_VALUE)
         params->hStdError = 0;
-    else if (VerifyConsoleIoHandle(console_handle_map(params->hStdError)))
+    else if (VerifyConsoleIoHandle(params->hStdError))
         params->hStdError = console_handle_map(params->hStdError);
 
     return TRUE;
@@ -1344,11 +1317,11 @@ BOOL WINAPI SetConsoleKeyShortcuts(BOOL set, BYTE keys, VOID *a, DWORD b)
 
 BOOL WINAPI GetCurrentConsoleFontEx(HANDLE hConsole, BOOL maxwindow, CONSOLE_FONT_INFOEX *fontinfo)
 {
-    BOOL ret;
+    DWORD size;
     struct
     {
-        unsigned int color_map[16];
-        WCHAR face_name[LF_FACESIZE];
+        struct condrv_output_info info;
+        WCHAR face_name[LF_FACESIZE - 1];
     } data;
 
     if (fontinfo->cbSize != sizeof(CONSOLE_FONT_INFOEX))
@@ -1357,37 +1330,30 @@ BOOL WINAPI GetCurrentConsoleFontEx(HANDLE hConsole, BOOL maxwindow, CONSOLE_FON
         return FALSE;
     }
 
-    SERVER_START_REQ(get_console_output_info)
+    if (!DeviceIoControl( hConsole, IOCTL_CONDRV_GET_OUTPUT_INFO, NULL, 0,
+                          &data, sizeof(data), &size, NULL ))
     {
-        req->handle = console_handle_unmap(hConsole);
-        wine_server_set_reply( req, &data, sizeof(data) - sizeof(WCHAR) );
-        if ((ret = !wine_server_call_err(req)))
-        {
-            fontinfo->nFont = 0;
-            if (maxwindow)
-            {
-                fontinfo->dwFontSize.X = min(reply->width, reply->max_width);
-                fontinfo->dwFontSize.Y = min(reply->height, reply->max_height);
-            }
-            else
-            {
-                fontinfo->dwFontSize.X = reply->win_right - reply->win_left + 1;
-                fontinfo->dwFontSize.Y = reply->win_bottom - reply->win_top + 1;
-            }
-            if (wine_server_reply_size( reply ) > sizeof(data.color_map))
-            {
-                data_size_t len = wine_server_reply_size( reply ) - sizeof(data.color_map);
-                memcpy( fontinfo->FaceName, data.face_name, len );
-                fontinfo->FaceName[len / sizeof(WCHAR)] = 0;
-            }
-            else
-                fontinfo->FaceName[0] = 0;
-            fontinfo->FontFamily = reply->font_pitch_family;
-            fontinfo->FontWeight = reply->font_weight;
-        }
+        SetLastError( ERROR_INVALID_HANDLE );
+        return FALSE;
     }
-    SERVER_END_REQ;
-    return ret;
+
+    fontinfo->nFont = 0;
+    if (maxwindow)
+    {
+        fontinfo->dwFontSize.X = min( data.info.width, data.info.max_width );
+        fontinfo->dwFontSize.Y = min( data.info.height, data.info.max_height );
+    }
+    else
+    {
+        fontinfo->dwFontSize.X = data.info.win_right - data.info.win_left + 1;
+        fontinfo->dwFontSize.Y = data.info.win_bottom - data.info.win_top + 1;
+    }
+    size -= sizeof(data.info);
+    if (size) memcpy( fontinfo->FaceName, data.face_name, size );
+    fontinfo->FaceName[size / sizeof(WCHAR)] = 0;
+    fontinfo->FontFamily = data.info.font_pitch_family;
+    fontinfo->FontWeight = data.info.font_weight;
+    return TRUE;
 }
 
 BOOL WINAPI GetCurrentConsoleFont(HANDLE hConsole, BOOL maxwindow, CONSOLE_FONT_INFO *fontinfo)
@@ -1409,6 +1375,7 @@ BOOL WINAPI GetCurrentConsoleFont(HANDLE hConsole, BOOL maxwindow, CONSOLE_FONT_
 
 static COORD get_console_font_size(HANDLE hConsole, DWORD index)
 {
+    struct condrv_output_info info;
     COORD c = {0,0};
 
     if (index >= GetNumberOfConsoleFonts())
@@ -1417,16 +1384,12 @@ static COORD get_console_font_size(HANDLE hConsole, DWORD index)
         return c;
     }
 
-    SERVER_START_REQ(get_console_output_info)
+    if (DeviceIoControl( hConsole, IOCTL_CONDRV_GET_OUTPUT_INFO, NULL, 0, &info, sizeof(info), NULL, NULL ))
     {
-        req->handle = console_handle_unmap(hConsole);
-        if (!wine_server_call_err(req))
-        {
-            c.X = reply->font_width;
-            c.Y = reply->font_height;
-        }
+        c.X = info.font_width;
+        c.Y = info.font_height;
     }
-    SERVER_END_REQ;
+    else SetLastError( ERROR_INVALID_HANDLE );
     return c;
 }
 
