@@ -36,6 +36,7 @@
 #include "winnls.h"
 #include "winerror.h"
 #include "wincon.h"
+#include "wine/condrv.h"
 #include "wine/server.h"
 #include "wine/exception.h"
 #include "wine/debug.h"
@@ -79,6 +80,28 @@ static inline obj_handle_t console_handle_unmap( HANDLE h )
     return wine_server_obj_handle( console_handle_map( h ) );
 }
 
+static BOOL console_ioctl( HANDLE handle, DWORD code, void *in_buff, DWORD in_count,
+                           void *out_buff, DWORD out_count, DWORD *read )
+{
+    IO_STATUS_BLOCK io;
+    NTSTATUS status;
+
+    status = NtDeviceIoControlFile( handle, NULL, NULL, NULL, &io, code, in_buff, in_count,
+                                    out_buff, out_count );
+    switch( status )
+    {
+    case STATUS_SUCCESS:
+        if (read) *read = io.Information;
+        return TRUE;
+    case STATUS_INVALID_PARAMETER:
+        break;
+    default:
+        status = STATUS_INVALID_HANDLE;
+        break;
+    }
+    return set_ntstatus( status );
+}
+
 /* map input records to ASCII */
 static void input_records_WtoA( INPUT_RECORD *buffer, int count )
 {
@@ -110,9 +133,8 @@ static void input_records_AtoW( INPUT_RECORD *buffer, int count )
 }
 
 /* map char infos to ASCII */
-static void char_info_WtoA( CHAR_INFO *buffer, int count )
+static void char_info_WtoA( UINT cp, CHAR_INFO *buffer, int count )
 {
-    UINT cp = GetConsoleOutputCP();
     char ch;
 
     while (count-- > 0)
@@ -140,36 +162,29 @@ static void char_info_AtoW( CHAR_INFO *buffer, int count )
 /* helper function for ScrollConsoleScreenBufferW */
 static void fill_console_output( HANDLE handle, int i, int j, int len, CHAR_INFO *fill )
 {
-    SERVER_START_REQ( fill_console_output )
-    {
-        req->handle    = console_handle_unmap( handle );
-        req->mode      = CHAR_INFO_MODE_TEXTATTR;
-        req->x         = i;
-        req->y         = j;
-        req->count     = len;
-        req->wrap      = FALSE;
-        req->data.ch   = fill->Char.UnicodeChar;
-        req->data.attr = fill->Attributes;
-        wine_server_call_err( req );
-    }
-    SERVER_END_REQ;
+    struct condrv_fill_output_params params;
+
+    params.mode  = CHAR_INFO_MODE_TEXTATTR;
+    params.x     = i;
+    params.y     = j;
+    params.count = len;
+    params.wrap  = FALSE;
+    params.ch    = fill->Char.UnicodeChar;
+    params.attr  = fill->Attributes;
+    console_ioctl( handle, IOCTL_CONDRV_FILL_OUTPUT, &params, sizeof(params), NULL, 0, NULL );
 }
 
 /* helper function for GetLargestConsoleWindowSize */
 static COORD get_largest_console_window_size( HANDLE handle )
 {
+    struct condrv_output_info info;
     COORD c = { 0, 0 };
 
-    SERVER_START_REQ( get_console_output_info )
-    {
-        req->handle = console_handle_unmap( handle );
-        if (!wine_server_call_err( req ))
-        {
-            c.X = reply->max_width;
-            c.Y = reply->max_height;
-        }
-    }
-    SERVER_END_REQ;
+    if (!console_ioctl( handle, IOCTL_CONDRV_GET_OUTPUT_INFO, NULL, 0, &info, sizeof(info), NULL ))
+        return c;
+
+    c.X = info.max_width;
+    c.Y = info.max_height;
     TRACE( "(%p), returning %dx%d\n", handle, c.X, c.Y );
     return c;
 }
@@ -381,7 +396,7 @@ DWORD WINAPI CtrlRoutine( void *arg )
 BOOL WINAPI DECLSPEC_HOTPATCH FillConsoleOutputAttribute( HANDLE handle, WORD attr, DWORD length,
                                                            COORD coord, DWORD *written )
 {
-    BOOL ret;
+    struct condrv_fill_output_params params;
 
     TRACE( "(%p,%d,%d,(%dx%d),%p)\n", handle, attr, length, coord.X, coord.Y, written );
 
@@ -392,19 +407,16 @@ BOOL WINAPI DECLSPEC_HOTPATCH FillConsoleOutputAttribute( HANDLE handle, WORD at
     }
 
     *written = 0;
-    SERVER_START_REQ( fill_console_output )
-    {
-        req->handle    = console_handle_unmap( handle );
-        req->x         = coord.X;
-        req->y         = coord.Y;
-        req->mode      = CHAR_INFO_MODE_ATTR;
-        req->wrap      = TRUE;
-        req->data.attr = attr;
-        req->count     = length;
-        if ((ret = !wine_server_call_err( req ))) *written = reply->written;
-    }
-    SERVER_END_REQ;
-    return ret;
+
+    params.mode  = CHAR_INFO_MODE_ATTR;
+    params.x     = coord.X;
+    params.y     = coord.Y;
+    params.count = length;
+    params.wrap  = TRUE;
+    params.ch    = 0;
+    params.attr  = attr;
+    return console_ioctl( handle, IOCTL_CONDRV_FILL_OUTPUT, &params, sizeof(params),
+                          written, sizeof(written), NULL );
 }
 
 
@@ -427,7 +439,7 @@ BOOL WINAPI DECLSPEC_HOTPATCH FillConsoleOutputCharacterA( HANDLE handle, CHAR c
 BOOL WINAPI DECLSPEC_HOTPATCH FillConsoleOutputCharacterW( HANDLE handle, WCHAR ch, DWORD length,
                                                            COORD coord, DWORD *written )
 {
-    BOOL ret;
+    struct condrv_fill_output_params params;
 
     TRACE( "(%p,%s,%d,(%dx%d),%p)\n", handle, debugstr_wn(&ch, 1), length, coord.X, coord.Y, written );
 
@@ -438,19 +450,16 @@ BOOL WINAPI DECLSPEC_HOTPATCH FillConsoleOutputCharacterW( HANDLE handle, WCHAR
     }
 
     *written = 0;
-    SERVER_START_REQ( fill_console_output )
-    {
-        req->handle  = console_handle_unmap( handle );
-        req->x       = coord.X;
-        req->y       = coord.Y;
-        req->mode    = CHAR_INFO_MODE_TEXT;
-        req->wrap    = TRUE;
-        req->data.ch = ch;
-        req->count   = length;
-        if ((ret = !wine_server_call_err( req ))) *written = reply->written;
-    }
-    SERVER_END_REQ;
-    return ret;
+
+    params.mode  = CHAR_INFO_MODE_TEXT;
+    params.x     = coord.X;
+    params.y     = coord.Y;
+    params.count = length;
+    params.wrap  = TRUE;
+    params.ch    = ch;
+    params.attr  = 0;
+    return console_ioctl( handle, IOCTL_CONDRV_FILL_OUTPUT, &params, sizeof(params),
+                          written, sizeof(written), NULL );
 }
 
 HANDLE get_console_wait_handle( HANDLE handle )
@@ -541,26 +550,19 @@ UINT WINAPI DECLSPEC_HOTPATCH GetConsoleCP(void)
  */
 BOOL WINAPI DECLSPEC_HOTPATCH GetConsoleCursorInfo( HANDLE handle, CONSOLE_CURSOR_INFO *info )
 {
-    BOOL ret;
+    struct condrv_output_info condrv_info;
 
-    SERVER_START_REQ( get_console_output_info )
-    {
-        req->handle = console_handle_unmap( handle );
-        ret = !wine_server_call_err( req );
-        if (ret && info)
-        {
-            info->dwSize = reply->cursor_size;
-            info->bVisible = reply->cursor_visible;
-        }
-    }
-    SERVER_END_REQ;
+    if (!console_ioctl( handle, IOCTL_CONDRV_GET_OUTPUT_INFO, NULL, 0, &condrv_info, sizeof(condrv_info), NULL ))
+        return FALSE;
 
-    if (!ret) return FALSE;
     if (!info)
     {
         SetLastError( ERROR_INVALID_ACCESS );
         return FALSE;
     }
+
+    info->dwSize   = condrv_info.cursor_size;
+    info->bVisible = condrv_info.cursor_visible;
     TRACE("(%p) returning (%d,%d)\n", handle, info->dwSize, info->bVisible);
     return TRUE;
 }
@@ -598,18 +600,7 @@ BOOL WINAPI DECLSPEC_HOTPATCH GetConsoleInputExeNameW( DWORD len, LPWSTR buffer
  */
 BOOL WINAPI DECLSPEC_HOTPATCH GetConsoleMode( HANDLE handle, DWORD *mode )
 {
-    BOOL ret;
-
-    SERVER_START_REQ( get_console_mode )
-    {
-        req->handle = console_handle_unmap( handle );
-        if ((ret = !wine_server_call_err( req )))
-        {
-            if (mode) *mode = reply->mode;
-        }
-    }
-    SERVER_END_REQ;
-    return ret;
+    return console_ioctl( handle, IOCTL_CONDRV_GET_MODE, NULL, 0, mode, sizeof(*mode), NULL );
 }
 
 
@@ -638,33 +629,29 @@ UINT WINAPI DECLSPEC_HOTPATCH GetConsoleOutputCP(void)
  */
 BOOL WINAPI DECLSPEC_HOTPATCH GetConsoleScreenBufferInfo( HANDLE handle, CONSOLE_SCREEN_BUFFER_INFO *info )
 {
-    BOOL ret;
+    struct condrv_output_info condrv_info;
 
-    SERVER_START_REQ( get_console_output_info )
-    {
-        req->handle = console_handle_unmap( handle );
-        if ((ret = !wine_server_call_err( req )))
-        {
-            info->dwSize.X              = reply->width;
-            info->dwSize.Y              = reply->height;
-            info->dwCursorPosition.X    = reply->cursor_x;
-            info->dwCursorPosition.Y    = reply->cursor_y;
-            info->wAttributes           = reply->attr;
-            info->srWindow.Left         = reply->win_left;
-            info->srWindow.Right        = reply->win_right;
-            info->srWindow.Top          = reply->win_top;
-            info->srWindow.Bottom       = reply->win_bottom;
-            info->dwMaximumWindowSize.X = min(reply->width, reply->max_width);
-            info->dwMaximumWindowSize.Y = min(reply->height, reply->max_height);
-        }
-    }
-    SERVER_END_REQ;
+    if (!console_ioctl( handle , IOCTL_CONDRV_GET_OUTPUT_INFO, NULL, 0,
+                        &condrv_info, sizeof(condrv_info), NULL ))
+        return FALSE;
+
+    info->dwSize.X              = condrv_info.width;
+    info->dwSize.Y              = condrv_info.height;
+    info->dwCursorPosition.X    = condrv_info.cursor_x;
+    info->dwCursorPosition.Y    = condrv_info.cursor_y;
+    info->wAttributes           = condrv_info.attr;
+    info->srWindow.Left         = condrv_info.win_left;
+    info->srWindow.Right        = condrv_info.win_right;
+    info->srWindow.Top          = condrv_info.win_top;
+    info->srWindow.Bottom       = condrv_info.win_bottom;
+    info->dwMaximumWindowSize.X = min(condrv_info.width, condrv_info.max_width);
+    info->dwMaximumWindowSize.Y = min(condrv_info.height, condrv_info.max_height);
 
     TRACE( "(%p,(%d,%d) (%d,%d) %d (%d,%d-%d,%d) (%d,%d)\n", handle,
            info->dwSize.X, info->dwSize.Y, info->dwCursorPosition.X, info->dwCursorPosition.Y,
-	  info->wAttributes, info->srWindow.Left, info->srWindow.Top, info->srWindow.Right,
+           info->wAttributes, info->srWindow.Left, info->srWindow.Top, info->srWindow.Right,
            info->srWindow.Bottom, info->dwMaximumWindowSize.X, info->dwMaximumWindowSize.Y );
-    return ret;
+    return TRUE;
 }
 
 
@@ -674,7 +661,7 @@ BOOL WINAPI DECLSPEC_HOTPATCH GetConsoleScreenBufferInfo( HANDLE handle, CONSOLE
 BOOL WINAPI DECLSPEC_HOTPATCH GetConsoleScreenBufferInfoEx( HANDLE handle,
                                                             CONSOLE_SCREEN_BUFFER_INFOEX *info )
 {
-    BOOL ret;
+    struct condrv_output_info condrv_info;
 
     if (info->cbSize != sizeof(CONSOLE_SCREEN_BUFFER_INFOEX))
     {
@@ -682,29 +669,25 @@ BOOL WINAPI DECLSPEC_HOTPATCH GetConsoleScreenBufferInfoEx( HANDLE handle,
         return FALSE;
     }
 
-    SERVER_START_REQ( get_console_output_info )
-    {
-        req->handle = console_handle_unmap( handle );
-        wine_server_set_reply( req, info->ColorTable, sizeof(info->ColorTable) );
-        if ((ret = !wine_server_call_err( req )))
-        {
-            info->dwSize.X              = reply->width;
-            info->dwSize.Y              = reply->height;
-            info->dwCursorPosition.X    = reply->cursor_x;
-            info->dwCursorPosition.Y    = reply->cursor_y;
-            info->wAttributes           = reply->attr;
-            info->srWindow.Left         = reply->win_left;
-            info->srWindow.Top          = reply->win_top;
-            info->srWindow.Right        = reply->win_right;
-            info->srWindow.Bottom       = reply->win_bottom;
-            info->dwMaximumWindowSize.X = min( reply->width, reply->max_width );
-            info->dwMaximumWindowSize.Y = min( reply->height, reply->max_height );
-            info->wPopupAttributes      = reply->popup_attr;
-            info->bFullscreenSupported  = FALSE;
-        }
-    }
-    SERVER_END_REQ;
-    return ret;
+    if (!console_ioctl( handle, IOCTL_CONDRV_GET_OUTPUT_INFO, NULL, 0, &condrv_info,
+                        sizeof(condrv_info), NULL ))
+        return FALSE;
+
+    info->dwSize.X              = condrv_info.width;
+    info->dwSize.Y              = condrv_info.height;
+    info->dwCursorPosition.X    = condrv_info.cursor_x;
+    info->dwCursorPosition.Y    = condrv_info.cursor_y;
+    info->wAttributes           = condrv_info.attr;
+    info->srWindow.Left         = condrv_info.win_left;
+    info->srWindow.Top          = condrv_info.win_top;
+    info->srWindow.Right        = condrv_info.win_right;
+    info->srWindow.Bottom       = condrv_info.win_bottom;
+    info->dwMaximumWindowSize.X = min( condrv_info.width, condrv_info.max_width );
+    info->dwMaximumWindowSize.Y = min( condrv_info.height, condrv_info.max_height );
+    info->wPopupAttributes      = condrv_info.popup_attr;
+    info->bFullscreenSupported  = FALSE;
+    memcpy( info->ColorTable, condrv_info.color_map, sizeof(info->ColorTable) );
+    return TRUE;
 }
 
 
@@ -760,24 +743,11 @@ COORD WINAPI DECLSPEC_HOTPATCH GetLargestConsoleWindowSize( HANDLE handle )
  */
 BOOL WINAPI DECLSPEC_HOTPATCH GetNumberOfConsoleInputEvents( HANDLE handle, DWORD *count )
 {
-    BOOL ret;
-
-    SERVER_START_REQ( read_console_input )
-    {
-        req->handle = console_handle_unmap( handle );
-        req->flush  = FALSE;
-        if ((ret = !wine_server_call_err( req )))
-        {
-            if (count) *count = reply->read;
-            else
-            {
-                SetLastError( ERROR_INVALID_ACCESS );
-                ret = FALSE;
-            }
-        }
-    }
-    SERVER_END_REQ;
-    return ret;
+    struct condrv_input_info info;
+    if (!console_ioctl( handle, IOCTL_CONDRV_GET_INPUT_INFO, NULL, 0, &info, sizeof(info), NULL ))
+        return FALSE;
+    *count = info.input_count;
+    return TRUE;
 }
 
 
@@ -802,20 +772,11 @@ BOOL WINAPI DECLSPEC_HOTPATCH PeekConsoleInputA( HANDLE handle, INPUT_RECORD *bu
 BOOL WINAPI DECLSPEC_HOTPATCH PeekConsoleInputW( HANDLE handle, INPUT_RECORD *buffer,
                                                  DWORD length, DWORD *count )
 {
-    BOOL ret;
-
-    SERVER_START_REQ( read_console_input )
-    {
-        req->handle = console_handle_unmap( handle );
-        req->flush  = FALSE;
-        wine_server_set_reply( req, buffer, length * sizeof(INPUT_RECORD) );
-        if ((ret = !wine_server_call_err( req )))
-        {
-            if (count) *count = length ? reply->read : 0;
-        }
-    }
-    SERVER_END_REQ;
-    return ret;
+    DWORD read;
+    if (!console_ioctl( handle, IOCTL_CONDRV_PEEK, NULL, 0, buffer, length * sizeof(*buffer), &read ))
+        return FALSE;
+    if (count) *count = read / sizeof(*buffer);
+    return TRUE;
 }
 
 
@@ -927,8 +888,9 @@ BOOL WINAPI DECLSPEC_HOTPATCH ReadConsoleOutputA( HANDLE handle, CHAR_INFO *buff
     ret = ReadConsoleOutputW( handle, buffer, size, coord, region );
     if (ret && region->Right >= region->Left)
     {
+        UINT cp = GetConsoleOutputCP();
         for (y = 0; y <= region->Bottom - region->Top; y++)
-            char_info_WtoA( &buffer[(coord.Y + y) * size.X + coord.X], region->Right - region->Left + 1 );
+            char_info_WtoA( cp, &buffer[(coord.Y + y) * size.X + coord.X], region->Right - region->Left + 1 );
     }
     return ret;
 }
@@ -1101,19 +1063,8 @@ BOOL WINAPI DECLSPEC_HOTPATCH ScrollConsoleScreenBufferW( HANDLE handle, SMALL_R
  */
 BOOL WINAPI DECLSPEC_HOTPATCH SetConsoleActiveScreenBuffer( HANDLE handle )
 {
-    BOOL ret;
-
     TRACE( "(%p)\n", handle );
-
-    SERVER_START_REQ( set_console_input_info )
-    {
-        req->handle    = 0;
-        req->mask      = SET_CONSOLE_INPUT_INFO_ACTIVE_SB;
-        req->active_sb = wine_server_obj_handle( handle );
-        ret = !wine_server_call_err( req );
-    }
-    SERVER_END_REQ;
-    return ret;
+    return console_ioctl( handle, IOCTL_CONDRV_ACTIVATE, NULL, 0, NULL, 0, NULL );
 }
 
 
@@ -1197,20 +1148,14 @@ BOOL WINAPI DECLSPEC_HOTPATCH SetConsoleCtrlHandler( PHANDLER_ROUTINE func, BOOL
  */
 BOOL WINAPI DECLSPEC_HOTPATCH SetConsoleCursorInfo( HANDLE handle, CONSOLE_CURSOR_INFO *info )
 {
-    BOOL ret;
+    struct condrv_output_info_params params = { SET_CONSOLE_OUTPUT_INFO_CURSOR_GEOM };
 
     TRACE( "(%p,%d,%d)\n", handle, info->dwSize, info->bVisible);
 
-    SERVER_START_REQ( set_console_output_info )
-    {
-        req->handle         = console_handle_unmap( handle );
-        req->cursor_size    = info->dwSize;
-        req->cursor_visible = info->bVisible;
-        req->mask           = SET_CONSOLE_OUTPUT_INFO_CURSOR_GEOM;
-        ret = !wine_server_call_err( req );
-    }
-    SERVER_END_REQ;
-    return ret;
+    params.info.cursor_size    = info->dwSize;
+    params.info.cursor_visible = info->bVisible;
+    return console_ioctl( handle, IOCTL_CONDRV_SET_OUTPUT_INFO, &params, sizeof(params),
+                          NULL, 0, NULL );
 }
 
 
@@ -1219,23 +1164,18 @@ BOOL WINAPI DECLSPEC_HOTPATCH SetConsoleCursorInfo( HANDLE handle, CONSOLE_CURSO
  */
 BOOL WINAPI DECLSPEC_HOTPATCH SetConsoleCursorPosition( HANDLE handle, COORD pos )
 {
+    struct condrv_output_info_params params = { SET_CONSOLE_OUTPUT_INFO_CURSOR_POS };
     CONSOLE_SCREEN_BUFFER_INFO info;
     int w, h, do_move = 0;
-    BOOL ret;
 
     TRACE( "%p %d %d\n", handle, pos.X, pos.Y );
 
-    SERVER_START_REQ( set_console_output_info )
-    {
-        req->handle         = console_handle_unmap( handle );
-        req->cursor_x       = pos.X;
-        req->cursor_y       = pos.Y;
-        req->mask           = SET_CONSOLE_OUTPUT_INFO_CURSOR_POS;
-        ret = !wine_server_call_err( req );
-    }
-    SERVER_END_REQ;
+    params.info.cursor_x = pos.X;
+    params.info.cursor_y = pos.Y;
+    if (!console_ioctl( handle, IOCTL_CONDRV_SET_OUTPUT_INFO, &params, sizeof(params), NULL, 0, NULL ))
+        return FALSE;
 
-    if (!ret || !GetConsoleScreenBufferInfo( handle, &info )) return FALSE;
+    if (!GetConsoleScreenBufferInfo( handle, &info )) return FALSE;
 
     /* if cursor is no longer visible, scroll the visible window... */
     w = info.srWindow.Right - info.srWindow.Left + 1;
@@ -1264,8 +1204,7 @@ BOOL WINAPI DECLSPEC_HOTPATCH SetConsoleCursorPosition( HANDLE handle, COORD pos
     }
     info.srWindow.Bottom = info.srWindow.Top + h - 1;
 
-    if (do_move) ret = SetConsoleWindowInfo( handle, TRUE, &info.srWindow );
-    return ret;
+    return !do_move || SetConsoleWindowInfo( handle, TRUE, &info.srWindow );
 }
 
 
@@ -1308,18 +1247,8 @@ BOOL WINAPI DECLSPEC_HOTPATCH SetConsoleInputExeNameW( LPCWSTR name )
  */
 BOOL WINAPI DECLSPEC_HOTPATCH SetConsoleMode( HANDLE handle, DWORD mode )
 {
-    BOOL ret;
-
-    SERVER_START_REQ(set_console_mode)
-    {
-	req->handle = console_handle_unmap( handle );
-	req->mode = mode;
-	ret = !wine_server_call_err( req );
-    }
-    SERVER_END_REQ;
-
-    TRACE( "(%p,%x) retval == %d\n", handle, mode, ret );
-    return ret;
+    TRACE( "(%p,%x)\n", handle, mode );
+    return console_ioctl( handle, IOCTL_CONDRV_SET_MODE, &mode, sizeof(mode), NULL, 0, NULL );
 }
 
 
@@ -1353,7 +1282,10 @@ BOOL WINAPI DECLSPEC_HOTPATCH SetConsoleOutputCP( UINT cp )
 BOOL WINAPI DECLSPEC_HOTPATCH SetConsoleScreenBufferInfoEx( HANDLE handle,
                                                             CONSOLE_SCREEN_BUFFER_INFOEX *info )
 {
-    BOOL ret;
+    struct condrv_output_info_params params =
+        { SET_CONSOLE_OUTPUT_INFO_CURSOR_POS | SET_CONSOLE_OUTPUT_INFO_SIZE |
+          SET_CONSOLE_OUTPUT_INFO_ATTR | SET_CONSOLE_OUTPUT_INFO_POPUP_ATTR |
+          SET_CONSOLE_OUTPUT_INFO_DISPLAY_WINDOW | SET_CONSOLE_OUTPUT_INFO_MAX_SIZE };
 
     TRACE("(%p, %p)\n", handle, info);
 
@@ -1363,29 +1295,19 @@ BOOL WINAPI DECLSPEC_HOTPATCH SetConsoleScreenBufferInfoEx( HANDLE handle,
         return FALSE;
     }
 
-    SERVER_START_REQ( set_console_output_info )
-    {
-        req->handle     = console_handle_unmap( handle );
-        req->mask       = SET_CONSOLE_OUTPUT_INFO_CURSOR_POS | SET_CONSOLE_OUTPUT_INFO_SIZE |
-                          SET_CONSOLE_OUTPUT_INFO_ATTR | SET_CONSOLE_OUTPUT_INFO_POPUP_ATTR |
-                          SET_CONSOLE_OUTPUT_INFO_DISPLAY_WINDOW | SET_CONSOLE_OUTPUT_INFO_MAX_SIZE;
-        req->width      = info->dwSize.X;
-        req->height     = info->dwSize.Y;
-        req->cursor_x   = info->dwCursorPosition.X;
-        req->cursor_y   = info->dwCursorPosition.Y;
-        req->attr       = info->wAttributes;
-        req->win_left   = info->srWindow.Left;
-        req->win_top    = info->srWindow.Top;
-        req->win_right  = info->srWindow.Right;
-        req->win_bottom = info->srWindow.Bottom;
-        req->popup_attr = info->wPopupAttributes;
-        req->max_width  = min( info->dwMaximumWindowSize.X, info->dwSize.X );
-        req->max_height = min( info->dwMaximumWindowSize.Y, info->dwSize.Y );
-        ret = !wine_server_call_err( req );
-    }
-    SERVER_END_REQ;
-
-    return ret;
+    params.info.width      = info->dwSize.X;
+    params.info.height     = info->dwSize.Y;
+    params.info.cursor_x   = info->dwCursorPosition.X;
+    params.info.cursor_y   = info->dwCursorPosition.Y;
+    params.info.attr       = info->wAttributes;
+    params.info.win_left   = info->srWindow.Left;
+    params.info.win_top    = info->srWindow.Top;
+    params.info.win_right  = info->srWindow.Right;
+    params.info.win_bottom = info->srWindow.Bottom;
+    params.info.popup_attr = info->wPopupAttributes;
+    params.info.max_width  = min( info->dwMaximumWindowSize.X, info->dwSize.X );
+    params.info.max_height = min( info->dwMaximumWindowSize.Y, info->dwSize.Y );
+    return console_ioctl( handle, IOCTL_CONDRV_SET_OUTPUT_INFO, &params, sizeof(params), NULL, 0, NULL );
 }
 
 
@@ -1394,19 +1316,13 @@ BOOL WINAPI DECLSPEC_HOTPATCH SetConsoleScreenBufferInfoEx( HANDLE handle,
  */
 BOOL WINAPI DECLSPEC_HOTPATCH SetConsoleScreenBufferSize( HANDLE handle, COORD size )
 {
-    BOOL ret;
+    struct condrv_output_info_params params = { SET_CONSOLE_OUTPUT_INFO_SIZE };
 
     TRACE( "(%p,(%d,%d))\n", handle, size.X, size.Y );
-    SERVER_START_REQ( set_console_output_info )
-    {
-        req->handle = console_handle_unmap( handle );
-        req->width  = size.X;
-        req->height = size.Y;
-        req->mask   = SET_CONSOLE_OUTPUT_INFO_SIZE;
-        ret = !wine_server_call_err( req );
-    }
-    SERVER_END_REQ;
-    return ret;
+
+    params.info.width  = size.X;
+    params.info.height = size.Y;
+    return console_ioctl( handle, IOCTL_CONDRV_SET_OUTPUT_INFO, &params, sizeof(params), NULL, 0, NULL );
 }
 
 
@@ -1415,18 +1331,12 @@ BOOL WINAPI DECLSPEC_HOTPATCH SetConsoleScreenBufferSize( HANDLE handle, COORD s
  */
 BOOL WINAPI DECLSPEC_HOTPATCH SetConsoleTextAttribute( HANDLE handle, WORD attr )
 {
-    BOOL ret;
+    struct condrv_output_info_params params = { SET_CONSOLE_OUTPUT_INFO_ATTR };
 
     TRACE( "(%p,%d)\n", handle, attr );
-    SERVER_START_REQ( set_console_output_info )
-    {
-        req->handle = console_handle_unmap( handle );
-        req->attr   = attr;
-        req->mask   = SET_CONSOLE_OUTPUT_INFO_ATTR;
-        ret = !wine_server_call_err( req );
-    }
-    SERVER_END_REQ;
-    return ret;
+
+    params.info.attr = attr;
+    return console_ioctl( handle, IOCTL_CONDRV_SET_OUTPUT_INFO, &params, sizeof(params), NULL, 0, NULL );
 }
 
 
@@ -1455,8 +1365,8 @@ BOOL WINAPI DECLSPEC_HOTPATCH SetConsoleTitleW( LPCWSTR title )
  */
 BOOL WINAPI DECLSPEC_HOTPATCH SetConsoleWindowInfo( HANDLE handle, BOOL absolute, SMALL_RECT *window )
 {
+    struct condrv_output_info_params params = { SET_CONSOLE_OUTPUT_INFO_DISPLAY_WINDOW };
     SMALL_RECT rect = *window;
-    BOOL ret;
 
     TRACE( "(%p,%d,(%d,%d-%d,%d))\n", handle, absolute, rect.Left, rect.Top, rect.Right, rect.Bottom );
 
@@ -1470,19 +1380,12 @@ BOOL WINAPI DECLSPEC_HOTPATCH SetConsoleWindowInfo( HANDLE handle, BOOL absolute
 	rect.Right  += info.srWindow.Right;
 	rect.Bottom += info.srWindow.Bottom;
     }
-    SERVER_START_REQ( set_console_output_info )
-    {
-        req->handle     = console_handle_unmap( handle );
-	req->win_left   = rect.Left;
-	req->win_top    = rect.Top;
-	req->win_right  = rect.Right;
-	req->win_bottom = rect.Bottom;
-        req->mask       = SET_CONSOLE_OUTPUT_INFO_DISPLAY_WINDOW;
-        ret = !wine_server_call_err( req );
-    }
-    SERVER_END_REQ;
 
-    return ret;
+    params.info.win_left   = rect.Left;
+    params.info.win_top    = rect.Top;
+    params.info.win_right  = rect.Right;
+    params.info.win_bottom = rect.Bottom;
+    return console_ioctl( handle, IOCTL_CONDRV_SET_OUTPUT_INFO, &params, sizeof(params), NULL, 0, NULL );
 }
 
 
@@ -1522,9 +1425,6 @@ BOOL WINAPI DECLSPEC_HOTPATCH WriteConsoleInputA( HANDLE handle, const INPUT_REC
 BOOL WINAPI DECLSPEC_HOTPATCH WriteConsoleInputW( HANDLE handle, const INPUT_RECORD *buffer,
                                                   DWORD count, DWORD *written )
 {
-    DWORD events_written = 0;
-    BOOL ret;
-
     TRACE( "(%p,%p,%d,%p)\n", handle, buffer, count, written );
 
     if (count > 0 && !buffer)
@@ -1532,21 +1432,17 @@ BOOL WINAPI DECLSPEC_HOTPATCH WriteConsoleInputW( HANDLE handle, const INPUT_REC
         SetLastError( ERROR_INVALID_ACCESS );
         return FALSE;
     }
-    SERVER_START_REQ( write_console_input )
-    {
-        req->handle = console_handle_unmap( handle );
-        wine_server_add_data( req, buffer, count * sizeof(INPUT_RECORD) );
-        if ((ret = !wine_server_call_err( req ))) events_written = reply->written;
-    }
-    SERVER_END_REQ;
 
-    if (written) *written = events_written;
-    else
+    if (!DeviceIoControl( handle, IOCTL_CONDRV_WRITE_INPUT, (void *)buffer, count * sizeof(*buffer), NULL, 0, NULL, NULL ))
+        return FALSE;
+
+    if (!written)
     {
         SetLastError( ERROR_INVALID_ACCESS );
-        ret = FALSE;
+        return FALSE;
     }
-    return ret;
+    *written = count;
+    return TRUE;
 }
 
 
@@ -1676,19 +1572,20 @@ BOOL WINAPI DECLSPEC_HOTPATCH WriteConsoleOutputCharacterA( HANDLE handle, LPCST
 
     if (length > 0)
     {
+        UINT cp = GetConsoleOutputCP();
         if (!str)
         {
             SetLastError( ERROR_INVALID_ACCESS );
             return FALSE;
         }
-        lenW = MultiByteToWideChar( GetConsoleOutputCP(), 0, str, length, NULL, 0 );
+        lenW = MultiByteToWideChar( cp, 0, str, length, NULL, 0 );
 
         if (!(strW = HeapAlloc( GetProcessHeap(), 0, lenW * sizeof(WCHAR) )))
         {
             SetLastError( ERROR_NOT_ENOUGH_MEMORY );
             return FALSE;
         }
-        MultiByteToWideChar( GetConsoleOutputCP(), 0, str, length, strW, lenW );
+        MultiByteToWideChar( cp, 0, str, length, strW, lenW );
     }
     ret = WriteConsoleOutputCharacterW( handle, strW, lenW, coord, written );
     HeapFree( GetProcessHeap(), 0, strW );
