@@ -301,7 +301,7 @@ DWORD WINAPI DECLSPEC_HOTPATCH GetModuleFileNameW( HMODULE module, LPWSTR filena
 {
     ULONG len = 0;
     ULONG_PTR magic;
-    LDR_MODULE *pldr;
+    LDR_DATA_TABLE_ENTRY *pldr;
     WIN16_SUBSYSTEM_TIB *win16_tib;
 
     if (!module && ((win16_tib = NtCurrentTeb()->Tib.SubSystemTib)) && win16_tib->exe_name)
@@ -544,6 +544,15 @@ HMODULE WINAPI /* DECLSPEC_HOTPATCH */ LoadPackagedLibrary( LPCWSTR name, DWORD
 }
 
 
+/***********************************************************************
+ *      LoadAppInitDlls    (kernelbase.@)
+ */
+void WINAPI LoadAppInitDlls(void)
+{
+    TRACE( "\n" );
+}
+
+
 /****************************************************************************
  *	RemoveDllDirectory   (kernelbase.@)
  */
