@@ -165,15 +165,6 @@ void raise_status( NTSTATUS status, EXCEPTION_RECORD *rec )
 }
 
 
-/*******************************************************************
- *		NtRaiseException (NTDLL.@)
- */
-NTSTATUS WINAPI NtRaiseException( EXCEPTION_RECORD *rec, CONTEXT *context, BOOL first_chance )
-{
-    return unix_funcs->NtRaiseException( rec, context, first_chance );
-}
-
-
 /***********************************************************************
  *            RtlRaiseStatus  (NTDLL.@)
  *
