@@ -564,9 +564,6 @@ NTSTATUS WINAPI dispatch_exception( EXCEPTION_RECORD *rec, CONTEXT *context )
               context->R12, context->R13, context->R14, context->R15 );
     }
 
-    /* fix up instruction pointer in context for EXCEPTION_BREAKPOINT */
-    if (rec->ExceptionCode == EXCEPTION_BREAKPOINT) context->Rip--;
-
     if (call_vectored_handlers( rec, context ) == EXCEPTION_CONTINUE_EXECUTION)
         NtContinue( context, FALSE );
 
@@ -588,22 +585,22 @@ __ASM_GLOBAL_FUNC( KiUserExceptionDispatcher,
                   "mov %rbp,-0x10(%rcx)\n\t"
                   "mov %rdi,-0x18(%rcx)\n\t"
                   "mov %rsi,-0x20(%rcx)\n\t"
-                  "lea -0x10(%rcx),%rbp\n\t"
+                  "lea -0x20(%rcx),%rbp\n\t"
                   "mov %rsp,%rdx\n\t" /* context */
                   "lea 0x4f0(%rsp),%rcx\n\t" /* rec */
                   __ASM_SEH(".seh_pushreg %rbp\n\t")
-                  __ASM_SEH(".seh_setframe %rbp,0\n\t")
                   __ASM_SEH(".seh_pushreg %rdi\n\t")
                   __ASM_SEH(".seh_pushreg %rsi\n\t")
+                  __ASM_SEH(".seh_setframe %rbp,0\n\t")
                   __ASM_SEH(".seh_endprologue\n\t")
 
                   __ASM_CFI(".cfi_signal_frame\n\t")
-                  __ASM_CFI(".cfi_adjust_cfa_offset 0x10\n\t")
-                  __ASM_CFI(".cfi_def_cfa %rbp,0x10\n\t")
-                  __ASM_CFI(".cfi_rel_offset %rip,0x8\n\t")
-                  __ASM_CFI(".cfi_rel_offset %rbp,0x0\n\t")
-                  __ASM_CFI(".cfi_rel_offset %rdi,-0x8\n\t")
-                  __ASM_CFI(".cfi_rel_offset %rsi,-0x10\n\t")
+                  __ASM_CFI(".cfi_adjust_cfa_offset 0x20\n\t")
+                  __ASM_CFI(".cfi_def_cfa %rbp,0x20\n\t")
+                  __ASM_CFI(".cfi_rel_offset %rip,0x18\n\t")
+                  __ASM_CFI(".cfi_rel_offset %rbp,0x10\n\t")
+                  __ASM_CFI(".cfi_rel_offset %rdi,0x8\n\t")
+                  __ASM_CFI(".cfi_rel_offset %rsi,0\n\t")
                    "call " __ASM_NAME("dispatch_exception") "\n\t"
                   "int3")
 
@@ -1338,7 +1335,12 @@ __ASM_GLOBAL_FUNC( RtlRaiseException,
                    "movq %rax,0xf8(%rdx)\n\t"   /* context->Rip */
                    "movq %rax,0x10(%rcx)\n\t"   /* rec->ExceptionAddress */
                    "movl $1,%r8d\n\t"
-                   "call " __ASM_NAME("NtRaiseException") "\n\t"
+                   "movq %gs:(0x30),%rax\n\t"   /* Teb */
+                   "movq 0x60(%rax),%rax\n\t"   /* Peb */
+                   "cmpb $0,0x02(%rax)\n\t"     /* BeingDebugged */
+                   "jne 1f\n\t"
+                   "call " __ASM_NAME("dispatch_exception") "\n"
+                   "1:\tcall " __ASM_NAME("NtRaiseException") "\n\t"
                    "movq %rax,%rcx\n\t"
                    "call " __ASM_NAME("RtlRaiseStatus") /* does not return */ );
 
