@@ -943,6 +943,7 @@ static void init_locale(void)
         if (preferred_langs) CFRelease( preferred_langs );
     }
 #endif
+    setlocale( LC_NUMERIC, "C" );  /* FIXME: oleaut32 depends on this */
 }
 
 
@@ -1056,16 +1057,14 @@ static void append_envW( WCHAR *env, SIZE_T *pos, const char *name, const WCHAR
 /* set an environment variable for one of the wine path variables */
 static void add_path_var( WCHAR *env, SIZE_T *pos, const char *name, const char *path )
 {
-    UNICODE_STRING nt_name;
-    ANSI_STRING unix_name;
+    WCHAR *nt_name;
 
     if (!path) append_envW( env, pos, name, NULL );
     else
     {
-        RtlInitAnsiString( &unix_name, path );
-        if (unix_to_nt_file_name( &unix_name, &nt_name )) return;
-        append_envW( env, pos, name, nt_name.Buffer );
-        RtlFreeUnicodeString( &nt_name );
+        if (unix_to_nt_file_name( path, &nt_name )) return;
+        append_envW( env, pos, name, nt_name );
+        free( nt_name );
     }
 }
 
@@ -1201,25 +1200,24 @@ void CDECL get_initial_directory( UNICODE_STRING *dir )
 
     if (pwd)
     {
-        ANSI_STRING unix_name;
-        UNICODE_STRING nt_name;
+        WCHAR *nt_name;
 
-        RtlInitAnsiString( &unix_name, pwd );
-        if (!unix_to_nt_file_name( &unix_name, &nt_name ))
+        if (!unix_to_nt_file_name( pwd, &nt_name ))
         {
             /* skip the \??\ prefix */
-            if (nt_name.Length > 6 * sizeof(WCHAR) && nt_name.Buffer[5] == ':')
+            ULONG len = wcslen( nt_name );
+            if (len > 6 && nt_name[5] == ':')
             {
-                dir->Length = nt_name.Length - 4 * sizeof(WCHAR);
-                memcpy( dir->Buffer, nt_name.Buffer + 4, dir->Length );
+                dir->Length = (len - 4) * sizeof(WCHAR);
+                memcpy( dir->Buffer, nt_name + 4, dir->Length );
             }
             else  /* change \??\ to \\?\ */
             {
-                dir->Length = nt_name.Length;
-                memcpy( dir->Buffer, nt_name.Buffer, dir->Length );
+                dir->Length = len * sizeof(WCHAR);
+                memcpy( dir->Buffer, nt_name, dir->Length );
                 dir->Buffer[1] = '\\';
             }
-            RtlFreeUnicodeString( &nt_name );
+            free( nt_name );
         }
     }
 
