@@ -104,6 +104,48 @@ typedef struct ucontext
 # define ERROR_sig(context)         REG_sig(error_code, context)
 # define TRAP_sig(context)          REG_sig(trap_no, context)
 
+struct extended_ctx
+{
+    unsigned long magic;
+    unsigned long size;
+};
+
+struct vfp_sigframe
+{
+    struct extended_ctx ctx;
+    unsigned long long fpregs[32];
+    unsigned long fpscr;
+};
+
+static void *get_extended_sigcontext( const ucontext_t *sigcontext, unsigned int magic )
+{
+    struct extended_ctx *ctx = (struct extended_ctx *)sigcontext->uc_regspace;
+    while ((char *)ctx < (char *)(sigcontext + 1) && ctx->magic && ctx->size)
+    {
+        if (ctx->magic == magic) return ctx;
+        ctx = (struct extended_ctx *)((char *)ctx + ctx->size);
+    }
+    return NULL;
+}
+
+static void save_fpu( CONTEXT *context, const ucontext_t *sigcontext )
+{
+    struct vfp_sigframe *frame = get_extended_sigcontext( sigcontext, 0x56465001 );
+
+    if (!frame) return;
+    memcpy( context->u.D, frame->fpregs, sizeof(context->u.D) );
+    context->Fpscr = frame->fpscr;
+}
+
+static void restore_fpu( const CONTEXT *context, ucontext_t *sigcontext )
+{
+    struct vfp_sigframe *frame = get_extended_sigcontext( sigcontext, 0x56465001 );
+
+    if (!frame) return;
+    memcpy( frame->fpregs, context->u.D, sizeof(context->u.D) );
+    frame->fpscr = context->Fpscr;
+}
+
 #elif defined(__FreeBSD__)
 
 /* All Registers access - only for local access */
@@ -117,6 +159,9 @@ typedef struct ucontext
 # define IP_sig(context)            REGn_sig(_REG_R12, context)   /* Intra-Procedure-call scratch register */
 # define FP_sig(context)            REGn_sig(_REG_FP, context)    /* Frame pointer */
 
+static void save_fpu( CONTEXT *context, const ucontext_t *sigcontext ) { }
+static void restore_fpu( const CONTEXT *context, ucontext_t *sigcontext ) { }
+
 #endif /* linux */
 
 enum arm_trap_code
@@ -127,8 +172,6 @@ enum arm_trap_code
     TRAP_ARM_ALIGNFLT   = 17,  /* Alignment check exception */
 };
 
-typedef void (WINAPI *raise_func)( EXCEPTION_RECORD *rec, CONTEXT *context );
-
 
 /***********************************************************************
  *           unwind_builtin_dll
@@ -200,6 +243,7 @@ static void save_context( CONTEXT *context, const ucontext_t *sigcontext )
     context->Cpsr = CPSR_sig(sigcontext); /* Current State Register */
     context->R11  = FP_sig(sigcontext);   /* Frame pointer */
     context->R12  = IP_sig(sigcontext);   /* Intra-Procedure-call scratch register */
+    save_fpu( context, sigcontext );
 }
 
 
@@ -216,33 +260,12 @@ static void restore_context( const CONTEXT *context, ucontext_t *sigcontext )
 #undef C
 
     SP_sig(sigcontext)   = context->Sp;   /* Stack pointer */
-    LR_sig(sigcontext)   = context->Lr ;  /* Link register */
+    LR_sig(sigcontext)   = context->Lr;   /* Link register */
     PC_sig(sigcontext)   = context->Pc;   /* Program Counter */
     CPSR_sig(sigcontext) = context->Cpsr; /* Current State Register */
     FP_sig(sigcontext)   = context->R11;  /* Frame pointer */
     IP_sig(sigcontext)   = context->R12;  /* Intra-Procedure-call scratch register */
-}
-
-
-/***********************************************************************
- *           save_fpu
- *
- * Set the FPU context from a sigcontext.
- */
-static inline void save_fpu( CONTEXT *context, const ucontext_t *sigcontext )
-{
-    FIXME("not implemented\n");
-}
-
-
-/***********************************************************************
- *           restore_fpu
- *
- * Restore the FPU context to a sigcontext.
- */
-static inline void restore_fpu( CONTEXT *context, const ucontext_t *sigcontext )
-{
-    FIXME("not implemented\n");
+    restore_fpu( context, sigcontext );
 }
 
 
@@ -482,42 +505,48 @@ NTSTATUS WINAPI NtGetContextThread( HANDLE handle, CONTEXT *context )
 }
 
 
-extern void raise_func_trampoline_thumb( EXCEPTION_RECORD *rec, CONTEXT *context, raise_func func );
+extern void raise_func_trampoline_thumb( EXCEPTION_RECORD *rec, CONTEXT *context, void *func );
 __ASM_GLOBAL_FUNC( raise_func_trampoline_thumb,
                    ".thumb\n\t"
-                   "blx r2\n\t"
+                   "bx r2\n\t"
                    "bkpt")
 
-extern void raise_func_trampoline_arm( EXCEPTION_RECORD *rec, CONTEXT *context, raise_func func );
+extern void raise_func_trampoline_arm( EXCEPTION_RECORD *rec, CONTEXT *context, void *func );
 __ASM_GLOBAL_FUNC( raise_func_trampoline_arm,
                    ".arm\n\t"
-                   "blx r2\n\t"
+                   "bx r2\n\t"
                    "bkpt")
 
 /***********************************************************************
- *           setup_exception_record
+ *           setup_exception
  *
- * Setup the exception record and context on the thread stack.
+ * Modify the signal context to call the exception raise function.
  */
-static EXCEPTION_RECORD *setup_exception( ucontext_t *sigcontext, raise_func func )
+static void setup_exception( ucontext_t *sigcontext, EXCEPTION_RECORD *rec )
 {
-    struct stack_layout
+    struct
     {
-        CONTEXT           context;
-        EXCEPTION_RECORD  rec;
+        CONTEXT          context;
+        EXCEPTION_RECORD rec;
     } *stack;
-    DWORD exception_code = 0;
 
-    stack = (struct stack_layout *)(SP_sig(sigcontext) & ~3);
-    stack--;  /* push the stack_layout structure */
+    void *stack_ptr = (void *)(SP_sig(sigcontext) & ~3);
+    CONTEXT context;
+    NTSTATUS status;
 
-    stack->rec.ExceptionRecord  = NULL;
-    stack->rec.ExceptionCode    = exception_code;
-    stack->rec.ExceptionFlags   = EXCEPTION_CONTINUABLE;
-    stack->rec.ExceptionAddress = (LPVOID)PC_sig(sigcontext);
-    stack->rec.NumberParameters = 0;
+    rec->ExceptionAddress = (void *)PC_sig(sigcontext);
+    save_context( &context, sigcontext );
 
-    save_context( &stack->context, sigcontext );
+    status = send_debug_event( rec, &context, TRUE );
+    if (status == DBG_CONTINUE || status == DBG_EXCEPTION_HANDLED)
+    {
+        restore_context( &context, sigcontext );
+        return;
+    }
+
+    stack = virtual_setup_exception( stack_ptr, sizeof(*stack), rec );
+    stack->rec = *rec;
+    stack->context = context;
 
     /* now modify the sigcontext to return to the raise function */
     SP_sig(sigcontext) = (DWORD)stack;
@@ -525,39 +554,15 @@ static EXCEPTION_RECORD *setup_exception( ucontext_t *sigcontext, raise_func fun
         PC_sig(sigcontext) = (DWORD)raise_func_trampoline_thumb;
     else
         PC_sig(sigcontext) = (DWORD)raise_func_trampoline_arm;
-    REGn_sig(0, sigcontext) = (DWORD)&stack->rec;  /* first arg for raise_func */
-    REGn_sig(1, sigcontext) = (DWORD)&stack->context; /* second arg for raise_func */
-    REGn_sig(2, sigcontext) = (DWORD)func; /* the raise_func as third arg for the trampoline */
-    return &stack->rec;
+    REGn_sig(0, sigcontext) = (DWORD)&stack->rec;  /* first arg for KiUserExceptionDispatcher */
+    REGn_sig(1, sigcontext) = (DWORD)&stack->context; /* second arg for KiUserExceptionDispatcher */
+    REGn_sig(2, sigcontext) = (DWORD)pKiUserExceptionDispatcher;
 }
 
-void WINAPI call_user_exception_dispatcher( EXCEPTION_RECORD *rec, CONTEXT *context )
+void WINAPI call_user_exception_dispatcher( EXCEPTION_RECORD *rec, CONTEXT *context,
+                                            NTSTATUS (WINAPI *dispatcher)(EXCEPTION_RECORD*,CONTEXT*) )
 {
-    pKiUserExceptionDispatcher( rec, context );
-}
-
-/**********************************************************************
- *		raise_segv_exception
- */
-static void WINAPI raise_segv_exception( EXCEPTION_RECORD *rec, CONTEXT *context )
-{
-    NTSTATUS status;
-
-    switch(rec->ExceptionCode)
-    {
-    case EXCEPTION_ACCESS_VIOLATION:
-        if (rec->NumberParameters == 2)
-        {
-            if (!(rec->ExceptionCode = virtual_handle_fault( (void *)rec->ExceptionInformation[1],
-                                                             rec->ExceptionInformation[0], FALSE )))
-                goto done;
-        }
-        break;
-    }
-    status = NtRaiseException( rec, context, TRUE );
-    if (status) RtlRaiseStatus( status );
-done:
-    set_cpu_context( context );
+    dispatcher( rec, context );
 }
 
 
@@ -566,53 +571,39 @@ done:
  *
  * Handler for SIGSEGV and related errors.
  */
-static void segv_handler( int signal, siginfo_t *info, void *ucontext )
+static void segv_handler( int signal, siginfo_t *siginfo, void *sigcontext )
 {
-    EXCEPTION_RECORD *rec;
-    ucontext_t *context = ucontext;
+    EXCEPTION_RECORD rec = { 0 };
+    ucontext_t *context = sigcontext;
 
-    /* check for page fault inside the thread stack */
-    if (get_trap_code(signal, context) == TRAP_ARM_PAGEFLT)
-    {
-        switch (virtual_handle_stack_fault( info->si_addr ))
-        {
-        case 1:  /* handled */
-            return;
-        case -1:  /* overflow */
-            rec = setup_exception( context, raise_segv_exception );
-            rec->ExceptionCode = EXCEPTION_STACK_OVERFLOW;
-            return;
-        }
-    }
-
-    rec = setup_exception( context, raise_segv_exception );
-    if (rec->ExceptionCode == EXCEPTION_STACK_OVERFLOW) return;
-
-    switch(get_trap_code(signal, context))
+    switch (get_trap_code(signal, context))
     {
     case TRAP_ARM_PRIVINFLT:   /* Invalid opcode exception */
-        rec->ExceptionCode = EXCEPTION_ILLEGAL_INSTRUCTION;
+        rec.ExceptionCode = EXCEPTION_ILLEGAL_INSTRUCTION;
         break;
     case TRAP_ARM_PAGEFLT:  /* Page fault */
-        rec->ExceptionCode = EXCEPTION_ACCESS_VIOLATION;
-        rec->NumberParameters = 2;
-        rec->ExceptionInformation[0] = (get_error_code(context) & 0x800) != 0;
-        rec->ExceptionInformation[1] = (ULONG_PTR)info->si_addr;
+        rec.NumberParameters = 2;
+        rec.ExceptionInformation[0] = (get_error_code(context) & 0x800) != 0;
+        rec.ExceptionInformation[1] = (ULONG_PTR)siginfo->si_addr;
+        rec.ExceptionCode = virtual_handle_fault( siginfo->si_addr, rec.ExceptionInformation[0],
+                                                  (void *)SP_sig(context) );
+        if (!rec.ExceptionCode) return;
         break;
     case TRAP_ARM_ALIGNFLT:  /* Alignment check exception */
-        rec->ExceptionCode = EXCEPTION_DATATYPE_MISALIGNMENT;
+        rec.ExceptionCode = EXCEPTION_DATATYPE_MISALIGNMENT;
         break;
     case TRAP_ARM_UNKNOWN:   /* Unknown fault code */
-        rec->ExceptionCode = EXCEPTION_ACCESS_VIOLATION;
-        rec->NumberParameters = 2;
-        rec->ExceptionInformation[0] = 0;
-        rec->ExceptionInformation[1] = 0xffffffff;
+        rec.ExceptionCode = EXCEPTION_ACCESS_VIOLATION;
+        rec.NumberParameters = 2;
+        rec.ExceptionInformation[0] = 0;
+        rec.ExceptionInformation[1] = 0xffffffff;
         break;
     default:
         ERR("Got unexpected trap %d\n", get_trap_code(signal, context));
-        rec->ExceptionCode = EXCEPTION_ILLEGAL_INSTRUCTION;
+        rec.ExceptionCode = EXCEPTION_ILLEGAL_INSTRUCTION;
         break;
     }
+    setup_exception( context, &rec );
 }
 
 
@@ -621,13 +612,11 @@ static void segv_handler( int signal, siginfo_t *info, void *ucontext )
  *
  * Handler for SIGTRAP.
  */
-static void trap_handler( int signal, siginfo_t *info, void *ucontext )
+static void trap_handler( int signal, siginfo_t *siginfo, void *sigcontext )
 {
-    EXCEPTION_RECORD rec;
-    CONTEXT context;
-    NTSTATUS status;
+    EXCEPTION_RECORD rec = { 0 };
 
-    switch ( info->si_code )
+    switch (siginfo->si_code)
     {
     case TRAP_TRACE:
         rec.ExceptionCode = EXCEPTION_SINGLE_STEP;
@@ -637,15 +626,7 @@ static void trap_handler( int signal, siginfo_t *info, void *ucontext )
         rec.ExceptionCode = EXCEPTION_BREAKPOINT;
         break;
     }
-
-    save_context( &context, ucontext );
-    rec.ExceptionFlags   = EXCEPTION_CONTINUABLE;
-    rec.ExceptionRecord  = NULL;
-    rec.ExceptionAddress = (LPVOID)context.Pc;
-    rec.NumberParameters = 0;
-    status = NtRaiseException( &rec, &context, TRUE );
-    if (status) RtlRaiseStatus( status );
-    restore_context( &context, ucontext );
+    setup_exception( sigcontext, &rec );
 }
 
 
@@ -656,12 +637,7 @@ static void trap_handler( int signal, siginfo_t *info, void *ucontext )
  */
 static void fpe_handler( int signal, siginfo_t *siginfo, void *sigcontext )
 {
-    EXCEPTION_RECORD rec;
-    CONTEXT context;
-    NTSTATUS status;
-
-    save_fpu( &context, sigcontext );
-    save_context( &context, sigcontext );
+    EXCEPTION_RECORD rec = { 0 };
 
     switch (siginfo->si_code & 0xffff )
     {
@@ -707,15 +683,7 @@ static void fpe_handler( int signal, siginfo_t *siginfo, void *sigcontext )
         rec.ExceptionCode = EXCEPTION_FLT_INVALID_OPERATION;
         break;
     }
-    rec.ExceptionFlags   = EXCEPTION_CONTINUABLE;
-    rec.ExceptionRecord  = NULL;
-    rec.ExceptionAddress = (LPVOID)context.Pc;
-    rec.NumberParameters = 0;
-    status = NtRaiseException( &rec, &context, TRUE );
-    if (status) RtlRaiseStatus( status );
-
-    restore_context( &context, sigcontext );
-    restore_fpu( &context, sigcontext );
+    setup_exception( sigcontext, &rec );
 }
 
 
@@ -726,19 +694,9 @@ static void fpe_handler( int signal, siginfo_t *siginfo, void *sigcontext )
  */
 static void int_handler( int signal, siginfo_t *siginfo, void *sigcontext )
 {
-    EXCEPTION_RECORD rec;
-    CONTEXT context;
-    NTSTATUS status;
+    EXCEPTION_RECORD rec = { CONTROL_C_EXIT };
 
-    save_context( &context, sigcontext );
-    rec.ExceptionCode    = CONTROL_C_EXIT;
-    rec.ExceptionFlags   = EXCEPTION_CONTINUABLE;
-    rec.ExceptionRecord  = NULL;
-    rec.ExceptionAddress = (LPVOID)context.Pc;
-    rec.NumberParameters = 0;
-    status = NtRaiseException( &rec, &context, TRUE );
-    if (status) RtlRaiseStatus( status );
-    restore_context( &context, sigcontext );
+    setup_exception( sigcontext, &rec );
 }
 
 
@@ -749,19 +707,9 @@ static void int_handler( int signal, siginfo_t *siginfo, void *sigcontext )
  */
 static void abrt_handler( int signal, siginfo_t *siginfo, void *sigcontext )
 {
-    EXCEPTION_RECORD rec;
-    CONTEXT context;
-    NTSTATUS status;
+    EXCEPTION_RECORD rec = { EXCEPTION_WINE_ASSERTION, EH_NONCONTINUABLE };
 
-    save_context( &context, sigcontext );
-    rec.ExceptionCode    = EXCEPTION_WINE_ASSERTION;
-    rec.ExceptionFlags   = EH_NONCONTINUABLE;
-    rec.ExceptionRecord  = NULL;
-    rec.ExceptionAddress = (LPVOID)context.Pc;
-    rec.NumberParameters = 0;
-    status = NtRaiseException( &rec, &context, TRUE );
-    if (status) RtlRaiseStatus( status );
-    restore_context( &context, sigcontext );
+    setup_exception( sigcontext, &rec );
 }
 
 
@@ -857,7 +805,7 @@ void signal_init_process(void)
     struct sigaction sig_act;
 
     sig_act.sa_mask = server_block_set;
-    sig_act.sa_flags = SA_RESTART | SA_SIGINFO;
+    sig_act.sa_flags = SA_RESTART | SA_SIGINFO | SA_ONSTACK;
 
     sig_act.sa_sigaction = int_handler;
     if (sigaction( SIGINT, &sig_act, NULL ) == -1) goto error;
