@@ -2082,7 +2082,10 @@ static void adapter_vk_init_driver_info(struct wined3d_adapter_vk *adapter_vk,
         gpu_description = &description;
     }
 
-    wined3d_driver_info_init(&adapter_vk->a.driver_info, gpu_description, vram_bytes, sysmem_bytes);
+    wined3d_driver_info_init(&adapter_vk->a.driver_info, gpu_description, adapter_vk->a.d3d_info.feature_level,
+            vram_bytes, sysmem_bytes);
+    TRACE("Reporting (fake) driver version 0x%08x-0x%08x.\n",
+            adapter_vk->a.driver_info.version_high, adapter_vk->a.driver_info.version_low);
 }
 
 static enum wined3d_feature_level feature_level_from_caps(const struct shader_caps *shader_caps)
@@ -2166,17 +2169,15 @@ static BOOL wined3d_adapter_vk_init(struct wined3d_adapter_vk *adapter_vk,
     struct wined3d_adapter *adapter = &adapter_vk->a;
     VkPhysicalDeviceIDProperties id_properties;
     VkPhysicalDeviceProperties2 properties2;
+    LUID *luid = NULL;
 
     TRACE("adapter_vk %p, ordinal %u, wined3d_creation_flags %#x.\n",
             adapter_vk, ordinal, wined3d_creation_flags);
 
-    if (!wined3d_adapter_init(adapter, ordinal, &wined3d_adapter_vk_ops))
-        return FALSE;
-
     if (!wined3d_init_vulkan(vk_info))
     {
         WARN("Failed to initialize Vulkan.\n");
-        goto fail;
+        return FALSE;
     }
 
     if (!(adapter_vk->physical_device = get_vulkan_physical_device(vk_info)))
@@ -2195,14 +2196,10 @@ static BOOL wined3d_adapter_vk_init(struct wined3d_adapter_vk *adapter_vk,
 
     VK_CALL(vkGetPhysicalDeviceMemoryProperties(adapter_vk->physical_device, &adapter_vk->memory_properties));
 
-    adapter_vk_init_driver_info(adapter_vk, &properties2.properties);
-    adapter->vram_bytes_used = 0;
-    TRACE("Emulating 0x%s bytes of video ram.\n", wine_dbgstr_longlong(adapter->driver_info.vram_bytes));
+    if (id_properties.deviceLUIDValid)
+        luid = (LUID *)id_properties.deviceLUID;
 
-    memcpy(&adapter->driver_uuid, id_properties.driverUUID, sizeof(adapter->driver_uuid));
-    memcpy(&adapter->device_uuid, id_properties.deviceUUID, sizeof(adapter->device_uuid));
-
-    if (!wined3d_adapter_vk_init_format_info(adapter_vk, vk_info))
+    if (!wined3d_adapter_init(adapter, ordinal, luid, &wined3d_adapter_vk_ops))
         goto fail_vulkan;
 
     adapter->vertex_pipe = wined3d_spirv_vertex_pipe_init_vk();
@@ -2212,13 +2209,23 @@ static BOOL wined3d_adapter_vk_init(struct wined3d_adapter_vk *adapter_vk,
 
     wined3d_adapter_vk_init_d3d_info(adapter_vk, wined3d_creation_flags);
 
+    adapter_vk_init_driver_info(adapter_vk, &properties2.properties);
+    adapter->vram_bytes_used = 0;
+    TRACE("Emulating 0x%s bytes of video ram.\n", wine_dbgstr_longlong(adapter->driver_info.vram_bytes));
+
+    memcpy(&adapter->driver_uuid, id_properties.driverUUID, sizeof(adapter->driver_uuid));
+    memcpy(&adapter->device_uuid, id_properties.deviceUUID, sizeof(adapter->device_uuid));
+
+    if (!wined3d_adapter_vk_init_format_info(adapter_vk, vk_info))
+        goto fail;
+
     return TRUE;
 
+fail:
+    wined3d_adapter_cleanup(adapter);
 fail_vulkan:
     VK_CALL(vkDestroyInstance(vk_info->instance, NULL));
     wined3d_unload_vulkan(vk_info);
-fail:
-    wined3d_adapter_cleanup(adapter);
     return FALSE;
 }
 
