@@ -253,7 +253,7 @@ static void	WCUSER_ComputePositions(struct inner_data* data)
     }
 
     SetWindowPos(data->hWnd, 0, 0, 0, r.right - r.left + dx, r.bottom - r.top + dy,
-		 SWP_NOMOVE|SWP_NOZORDER);
+                 SWP_NOMOVE|SWP_NOZORDER|SWP_NOACTIVATE);
     WCUSER_ShapeCursor(data, data->curcfg.cursor_size, data->curcfg.cursor_visible, TRUE);
     WCUSER_PosCursor(data);
 }
@@ -1365,7 +1365,7 @@ static int WCUSER_MainLoop(struct inner_data* data)
     ShowWindow(data->hWnd, data->nCmdShow);
     while (!data->dying || !data->curcfg.exit_on_die)
     {
-	switch (MsgWaitForMultipleObjects(1, &data->hSynchro, FALSE, INFINITE, QS_ALLINPUT))
+	switch (MsgWaitForMultipleObjects(1, &data->overlapped.hEvent, FALSE, INFINITE, QS_ALLINPUT))
 	{
 	case WAIT_OBJECT_0:
 	    WINECON_GrabChanges(data);
